import pytest

from yatbaf.dispatcher import Dispatcher
from yatbaf.enums import UpdateType
from yatbaf.exceptions import BotWarning
from yatbaf.router import OnEditedChannelPost
from yatbaf.router import OnEditedMessage
from yatbaf.router import OnMessage
from yatbaf.router import OnPoll


def test_routers(token):
    dispatcher = Dispatcher(token)
    for type_ in UpdateType:
        dispatcher._routers[type_]


def test_add_router(token, router):
    dispatcher = Dispatcher(token, routers=[router])
    assert len(routers := dispatcher._routers[router._update_type]) == 1
    assert routers[0] is router
    assert router._parent is dispatcher


def test_add_router_duplicate_same_object(token):
    router = OnMessage()
    with pytest.warns(BotWarning):
        dispatcher = Dispatcher(
            token,
            routers=[
                router,
                router,
            ],
        )
    assert len(routers := dispatcher._routers[router._update_type]) == 1
    assert router in routers


def test_add_router_duplicate_equal_objects(token):
    router1 = OnMessage(name="router1")
    router2 = OnMessage(name="router1")
    with pytest.warns(BotWarning):
        dispatcher = Dispatcher(
            token,
            routers=[
                router1,
                router2,
            ],
        )
    assert len(routers := dispatcher._routers["message"]) == 1
    assert routers[0] is router1


def test_add_routers(token):
    dispatcher = Dispatcher(
        token,
        routers=[
            message := OnMessage(),
            poll := OnPoll(),
            edited_message := OnEditedMessage(),
            edited_post := OnEditedChannelPost(),
        ],
    )
    assert message in dispatcher._routers["message"]
    assert poll in dispatcher._routers["poll"]
    assert edited_message in dispatcher._routers["edited_message"]
    assert edited_post in dispatcher._routers["edited_channel_post"]


def test_add_router_registered(token):
    router = OnMessage()
    nested = OnMessage()
    router.add_router(nested)
    with pytest.raises(ValueError):
        Dispatcher(
            token,
            routers=[
                router,
                nested,
            ],
        )



"""Telegram Bot API Methods."""

__all__ = [
    "AddStickerToSet",
    "AnswerCallbackQuery",
    "AnswerInlineQuery",
    "AnswerPreCheckoutQuery",
    "AnswerShippingQuery",
    "AnswerWebAppQuery",
    "ApproveChatJoinRequest",
    "BanChatMember",
    "BanChatSenderChat",
    "Close",
    "CloseForumTopic",
    "CloseGeneralForumTopic",
    "CopyMessage",
    "CopyMessages",
    "CreateChatInviteLink",
    "CreateForumTopic",
    "CreateInvoiceLink",
    "CreateNewStickerSet",
    "DeclineChatJoinRequest",
    "DeleteChatPhoto",
    "DeleteChatStickerSet",
    "DeleteForumTopic",
    "DeleteMessage",
    "DeleteMessages",
    "DeleteMyCommands",
    "DeleteStickerFromSet",
    "DeleteStickerSet",
    "DeleteWebhook",
    "EditChatInviteLink",
    "EditForumTopic",
    "EditGeneralForumTopic",
    "EditMessageCaption",
    "EditMessageLiveLocation",
    "EditMessageMedia",
    "EditMessageReplyMarkup",
    "EditMessageText",
    "ExportChatInviteLink",
    "ForwardMessage",
    "ForwardMessages",
    "GetChat",
    "GetChatAdministrators",
    "GetChatMember",
    "GetChatMemberCount",
    "GetChatMenuButton",
    "GetCustomEmojiStickers",
    "GetFile",
    "GetForumTopicIconStickers",
    "GetGameHighScores",
    "GetMe",
    "GetMyCommands",
    "GetMyDefaultAdministratorRights",
    "GetMyDescription",
    "GetMyName",
    "GetMyShortDescription",
    "GetStickerSet",
    "GetUpdates",
    "GetUserProfilePhotos",
    "GetUserChatBoosts",
    "GetWebhookInfo",
    "HideGeneralForumTopic",
    "LeaveChat",
    "LogOut",
    "PinChatMessage",
    "PromoteChatMember",
    "ReopenForumTopic",
    "ReopenGeneralForumTopic",
    "RestrictChatMember",
    "RevokeChatInviteLink",
    "SendAnimation",
    "SendAudio",
    "SendChatAction",
    "SendContact",
    "SendDice",
    "SendDocument",
    "SendGame",
    "SendInvoice",
    "SendLocation",
    "SendMediaGroup",
    "SendMessage",
    "SendPhoto",
    "SendPoll",
    "SendSticker",
    "SendVenue",
    "SendVideo",
    "SendVideoNote",
    "SendVoice",
    "SetChatAdministratorCustomTitle",
    "SetChatDescription",
    "SetChatMenuButton",
    "SetChatPermissions",
    "SetChatPhoto",
    "SetChatStickerSet",
    "SetChatTitle",
    "SetCustomEmojiStickerSetThumbnail",
    "SetGameScore",
    "SetMessageReaction",
    "SetMyCommands",
    "SetMyDefaultAdministratorRights",
    "SetMyDescription",
    "SetMyName",
    "SetMyShortDescription",
    "SetPassportDataErrors",
    "SetStickerEmojiList",
    "SetStickerKeywords",
    "SetStickerMaskPosition",
    "SetStickerPositionInSet",
    "SetStickerSetThumbnail",
    "SetStickerSetTitle",
    "SetWebhook",
    "StopMessageLiveLocation",
    "StopPoll",
    "UnbanChatMember",
    "UnbanChatSenderChat",
    "UnhideGeneralForumTopic",
    "UnpinAllChatMessages",
    "UnpinAllForumTopicMessages",
    "UnpinAllGeneralForumTopicMessages",
    "UnpinChatMessage",
    "UploadStickerFile",
]

from .add_sticker_to_set import AddStickerToSet
from .answer_callback_query import AnswerCallbackQuery
from .answer_inline_query import AnswerInlineQuery
from .answer_pre_checkout_query import AnswerPreCheckoutQuery
from .answer_shipping_query import AnswerShippingQuery
from .answer_web_app_query import AnswerWebAppQuery
from .approve_chat_join_request import ApproveChatJoinRequest
from .ban_chat_member import BanChatMember
from .ban_chat_sender_chat import BanChatSenderChat
from .close import Close
from .close_forum_topic import CloseForumTopic
from .close_general_forum_topic import CloseGeneralForumTopic
from .copy_message import CopyMessage
from .copy_messages import CopyMessages
from .create_chat_invite_link import CreateChatInviteLink
from .create_forum_topic import CreateForumTopic
from .create_invoice_link import CreateInvoiceLink
from .create_new_sticker_set import CreateNewStickerSet
from .decline_chat_join_request import DeclineChatJoinRequest
from .delete_chat_photo import DeleteChatPhoto
from .delete_chat_sticker_set import DeleteChatStickerSet
from .delete_forum_topic import DeleteForumTopic
from .delete_message import DeleteMessage
from .delete_messages import DeleteMessages
from .delete_my_commands import DeleteMyCommands
from .delete_sticker_from_set import DeleteStickerFromSet
from .delete_sticker_set import DeleteStickerSet
from .delete_webhook import DeleteWebhook
from .edit_chat_invite_link import EditChatInviteLink
from .edit_forum_topic import EditForumTopic
from .edit_general_forum_topic import EditGeneralForumTopic
from .edit_message_caption import EditMessageCaption
from .edit_message_live_location import EditMessageLiveLocation
from .edit_message_media import EditMessageMedia
from .edit_message_reply_markup import EditMessageReplyMarkup
from .edit_message_text import EditMessageText
from .export_chat_invite_link import ExportChatInviteLink
from .forward_message import ForwardMessage
from .forward_messages import ForwardMessages
from .get_chat import GetChat
from .get_chat_administrators import GetChatAdministrators
from .get_chat_member import GetChatMember
from .get_chat_member_count import GetChatMemberCount
from .get_chat_menu_button import GetChatMenuButton
from .get_custom_emoji_stickers import GetCustomEmojiStickers
from .get_file import GetFile
from .get_forum_topic_icon_stickers import GetForumTopicIconStickers
from .get_game_high_scores import GetGameHighScores
from .get_me import GetMe
from .get_my_commands import GetMyCommands
from .get_my_default_administrator_rights import \
    GetMyDefaultAdministratorRights
from .get_my_description import GetMyDescription
from .get_my_name import GetMyName
from .get_my_short_description import GetMyShortDescription
from .get_sticker_set import GetStickerSet
from .get_updates import GetUpdates
from .get_user_chat_boosts import GetUserChatBoosts
from .get_user_profile_photos import GetUserProfilePhotos
from .get_webhook_info import GetWebhookInfo
from .hide_general_forum_topic import HideGeneralForumTopic
from .leave_chat import LeaveChat
from .log_out import LogOut
from .pin_chat_message import PinChatMessage
from .promote_chat_member import PromoteChatMember
from .reopen_forum_topic import ReopenForumTopic
from .reopen_general_forum_topic import ReopenGeneralForumTopic
from .restrict_chat_member import RestrictChatMember
from .revoke_chat_invite_link import RevokeChatInviteLink
from .send_animation import SendAnimation
from .send_audio import SendAudio
from .send_chat_action import SendChatAction
from .send_contact import SendContact
from .send_dice import SendDice
from .send_document import SendDocument
from .send_game import SendGame
from .send_invoice import SendInvoice
from .send_location import SendLocation
from .send_media_group import SendMediaGroup
from .send_message import SendMessage
from .send_photo import SendPhoto
from .send_poll import SendPoll
from .send_sticker import SendSticker
from .send_venue import SendVenue
from .send_video import SendVideo
from .send_video_note import SendVideoNote
from .send_voice import SendVoice
from .set_chat_administrator_custom_title import \
    SetChatAdministratorCustomTitle
from .set_chat_description import SetChatDescription
from .set_chat_menu_button import SetChatMenuButton
from .set_chat_permissions import SetChatPermissions
from .set_chat_photo import SetChatPhoto
from .set_chat_sticker_set import SetChatStickerSet
from .set_chat_title import SetChatTitle
from .set_custom_emoji_sticker_set_thumbnail import \
    SetCustomEmojiStickerSetThumbnail
from .set_game_score import SetGameScore
from .set_message_reaction import SetMessageReaction
from .set_my_commands import SetMyCommands
from .set_my_default_administrator_rights import \
    SetMyDefaultAdministratorRights
from .set_my_description import SetMyDescription
from .set_my_name import SetMyName
from .set_my_short_description import SetMyShortDescription
from .set_passport_data_errors import SetPassportDataErrors
from .set_sticker_emoji_list import SetStickerEmojiList
from .set_sticker_keywords import SetStickerKeywords
from .set_sticker_mask_position import SetStickerMaskPosition
from .set_sticker_position_in_set import SetStickerPositionInSet
from .set_sticker_set_thumbnail import SetStickerSetThumbnail
from .set_sticker_set_title import SetStickerSetTitle
from .set_webhook import SetWebhook
from .stop_message_live_location import StopMessageLiveLocation
from .stop_poll import StopPoll
from .unban_chat_member import UnbanChatMember
from .unban_chat_sender_chat import UnbanChatSenderChat
from .unhide_general_forum_topic import UnhideGeneralForumTopic
from .unpin_all_chat_messages import UnpinAllChatMessages
from .unpin_all_forum_topic_messages import UnpinAllForumTopicMessages
from .unpin_all_general_forum_topic_messages import \
    UnpinAllGeneralForumTopicMessages
from .unpin_chat_message import UnpinChatMessage
from .upload_sticker_file import UploadStickerFile



from __future__ import annotations

from typing import final

from .abc import TelegramType
from .chat import Chat
from .user import User


@final
class PollAnswer(TelegramType):
    """This object represents an answer of a user in a non-anonymous poll.

    See: https://core.telegram.org/bots/api#pollanswer
    """

    poll_id: str
    """Unique poll identifier."""

    option_ids: list[int]
    """0-based identifiers of answer options, chosen by the user. May be empty
    if the user retracted their vote.
    """

    voter_chat: Chat | None = None
    """*Optional.* The chat that changed the answer to the poll, if the voter
    is anonymous.
    """

    user: User | None = None
    """*Optional.* The user that changed the answer to the poll, if the voter
    isn't anonymous.
    """
